#!/usr/bin/env python
# -*- coding: utf-8 -*-

import curses

python_string = \
"""Python is a general-purpose, high-level programming language whose design
philosophy emphasizes code readability. Python's syntax allows programmers to
express concepts in fewer lines of code than would be possible in languages
such as C, and the language provides constructs intended to enable clear
programs on both a small and large scale.

Python supports multiple programming paradigms, including object-oriented,
imperative and functional programming styles. It features a fully dynamic
type system and automatic memory management and has a large and comprehensive
standard library.

Like other dynamic languages, Python is often used as a scripting language,
but is also used in a wide range of non-scripting contexts. Using third-party
tools, Python code can be packaged into standalone executable programs. Python
interpreters are available for many operating systems.

CPython, the reference implementation of Python, is free and open source
software and has a community-based development model, as do nearly all of its
alternative implementations. CPython is managed by the non-profit Python
Software Foundation."""

def main(stdscr):
	curses.curs_set(False)
	stdscr.refresh()

	stdscr.addstr(1,5,"Use the cursor keys to move the text ('q' to quit)", curses.A_BOLD)

	win = curses.newwin(16,60,3,5)
	win.border()

	padmaxy, padmaxx = 20, 80
	pad = curses.newpad(padmaxy, padmaxx)
	pad.addstr(python_string)
	pady, padx = 0, 0

	win.noutrefresh()
	pad.noutrefresh(pady, padx, 4, 7, 4+13, 7+55)
	curses.doupdate()

	while True:
		c = stdscr.getch()
		if c == ord('q'): break
		if c == curses.KEY_UP: pady = pady-1 if pady > 0 else 0
		if c == curses.KEY_DOWN: pady = pady+1 if pady < padmaxy-1-13 else padmaxy-1-13
		if c == curses.KEY_LEFT: padx = padx-1 if padx > 0 else 0
		if c == curses.KEY_RIGHT: padx = padx+1 if padx < padmaxx-1-55 else padmaxx-1-55
		
		pad.refresh(pady, padx, 4, 7, 4+13, 7+55)
	
	return
	
if __name__ == "__main__":
	curses.wrapper(main)
	
