package org.codeproject.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.codeproject.domain.Comment;
import org.codeproject.domain.Post;

public class CommentDao extends DaoBase<Comment> {
	
	private static CommentMapper commentMapper = new CommentMapper();
	
	private static final String SQL_INSERT = "INSERT INTO comment(postId, content) VALUES(?,?)";
	private static final String SQL_UPDATE = "UPDATE comment SET content = ? WHERE id = ?";
	private static final String SQL_GETBYPOSTID = "SELECT * FROM comment WHERE postId = ?";
	
	List<Comment> getByPost(Post post, Connection connection) throws Exception {
		List<Comment> comments = new ArrayList<Comment>();
		
		PreparedStatement selectStatement = connection.prepareStatement(SQL_GETBYPOSTID);
		selectStatement.setLong(1, post.getId());
		ResultSet resultSet = selectStatement.executeQuery();
	
		while (resultSet.next()) {
			 comments.add(commentMapper.mapRow(resultSet, connection));
		}
	
		return comments;
	}

	@Override
	void save(Comment comment, Connection connection) throws Exception {
		PreparedStatement insertStatement = connection.prepareStatement(SQL_INSERT, Statement.RETURN_GENERATED_KEYS);
		insertStatement.setLong(1, comment.getPostId());
		insertStatement.setString(2, comment.getContent());
		insertStatement.executeUpdate();
		
		ResultSet resultSet = insertStatement.getGeneratedKeys();
		if (resultSet.next()) {
			Long key = resultSet.getLong(1);
			comment.setId(key);
		}
		else {
			throw new DaoException("Cannot get inserted comment id!");
		}
	}

	@Override
	void update(Comment comment, Connection connection) throws Exception {
		PreparedStatement updateStatement = connection.prepareStatement(SQL_UPDATE);
		updateStatement.setString(1, comment.getContent());
		updateStatement.setLong(2, comment.getId());
		updateStatement.executeUpdate();
	}
	
}