package org.codeproject.dao;

import java.lang.reflect.ParameterizedType;
import java.sql.Connection;
import java.sql.PreparedStatement;

import org.codeproject.domain.DomainBase;

abstract class DaoBase<T extends DomainBase> {
	
	private Class<T> persistentClass;

	private static final String SQL_DELETE = "DELETE FROM %s WHERE id = ?";
	
	@SuppressWarnings("unchecked")
	public DaoBase() {
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
	}
	
	abstract void save(T object, Connection connection) throws Exception;
	abstract void update(T object, Connection connection) throws Exception;
	
	void delete(T object, Connection connection) throws Exception {
		String sqlStatement = String.format(SQL_DELETE, persistentClass.getSimpleName().toString().toLowerCase());
		
		PreparedStatement selectStatement = connection.prepareStatement(sqlStatement);
		selectStatement.setLong(1, object.getId());
		selectStatement.executeUpdate();
	}
	
}
