package org.codeproject.dao;

import java.lang.reflect.ParameterizedType;
import java.sql.Connection;
import java.sql.ResultSet;

import org.codeproject.domain.DomainBase;

abstract class DataMapper<T extends DomainBase> {
	
	private Class<T> persistentClass;
	
	@SuppressWarnings("unchecked")
	public DataMapper() {
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
	}
	
	public T mapRow(ResultSet resultSet, Connection connection) throws Exception {
		T resultObject = persistentClass.newInstance();
		
		resultObject.setId(resultSet.getLong("id"));
		resultObject.setCreationDate(resultSet.getTimestamp("creationDate"));
		resultObject.setLastModified(resultSet.getTimestamp("lastModified"));
		
		mapRowParts(resultObject, resultSet, connection);
		
		return resultObject;
	}
	
	protected abstract void mapRowParts(T element, ResultSet resultSet, Connection connection) throws Exception;
	
}