package org.codeproject.dao;

import static java.lang.System.out;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertNull;
import static junit.framework.Assert.assertNotNull;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;

import org.codeproject.domain.DomainBase;

public class PersistenceManager {

	private List<DomainBase> newObjects = new ArrayList<DomainBase>();
	private List<DomainBase> dirtyObjects = new ArrayList<DomainBase>();
	private List<DomainBase> removedObjects = new ArrayList<DomainBase>();
	
	public void add(DomainBase object) {
		assertNull("object id not null", object.getId());
		assertFalse("object dirty", dirtyObjects.contains(object));
		assertFalse("object removed", removedObjects.contains(object));
		assertFalse("object already added", newObjects.contains(object));
		
		newObjects.add(object);
	}
	
	public void update(DomainBase object) {
		assertNotNull("object id not null", object.getId());
		assertFalse("object removed", removedObjects.contains(object));
		
		if (!dirtyObjects.contains(object) && !newObjects.contains(object)) {
			dirtyObjects.add(object);
		}
	}
	
	public void delete(DomainBase object) {
		assertNotNull("object id not null", object.getId());
		
		if (newObjects.remove(object)) return;
		
		dirtyObjects.remove(object);
		
		if (!removedObjects.contains(object)) {
			removedObjects.add(object);
		}
	}
	
	public void commit() throws Exception {
		Long millisBefore = System.currentTimeMillis();
		Connection connection = DatabaseConnectionMgr.getConnection();
		out.println(String.format("got connection in %sms", System.currentTimeMillis() - millisBefore));
		
		try {
			connection.setAutoCommit(false);
			
			insertNew(connection);
			updateDirty(connection);
			deleteRemoved(connection);
			
			connection.commit();
		}
		finally {
			connection.close();
		}
	}
	
	@SuppressWarnings("unchecked")
	private void insertNew(Connection connection) throws Exception {
		for (DomainBase object : newObjects) {
			DaoBase<DomainBase> dao = DaoRegistry.instance().getDao(object.getClass());
			dao.save(object, connection);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void updateDirty(Connection connection) throws Exception {
		for (DomainBase object : dirtyObjects) {
			DaoBase<DomainBase> dao = DaoRegistry.instance().getDao(object.getClass());
			dao.update(object, connection);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void deleteRemoved(Connection connection) throws Exception {
		for (DomainBase object : removedObjects) {
			DaoBase<DomainBase> dao = DaoRegistry.instance().getDao(object.getClass());
			dao.delete(object, connection);
		}
	}
	
}
