package org.codeproject.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.codeproject.domain.Comment;
import org.codeproject.domain.Post;

public class PostDao extends DaoBase<Post> {
	
	private static PostMapper postMapper = new PostMapper();
	
	private static final String SQL_INSERT = "INSERT INTO post(title, content) VALUES(?,?)";
	private static final String SQL_UPDATE = "UPDATE post SET title = ?, content = ? WHERE id = ?";
	private static final String SQL_GETBYID = "SELECT * FROM post WHERE id = ?";
	private static final String SQL_GETALL = "SELECT * FROM post";
	
	private static CommentDao commentDao = (CommentDao) DaoRegistry.instance().getDao(Comment.class);
	
	public Post getById(Long id) throws Exception {
		Connection currentConnection = DatabaseConnectionMgr.getConnection();
		
		try {
			PreparedStatement selectStatement = currentConnection.prepareStatement(SQL_GETBYID);
			selectStatement.setLong(1, id);
			ResultSet resultSet = selectStatement.executeQuery();
		
			if (resultSet.next()) {
				return postMapper.mapRow(resultSet, currentConnection);
			}
		}
		finally {
			currentConnection.close();
		}
		
		return null;
	}
	
	public List<Post> getAll() throws Exception {
		Connection currentConnection = DatabaseConnectionMgr.getConnection();

		List<Post> posts = new ArrayList<Post>();
		try {
			PreparedStatement selectStatement = currentConnection.prepareStatement(SQL_GETALL);
			ResultSet resultSet = selectStatement.executeQuery();

			while(resultSet.next()) {
				posts.add(postMapper.mapRow(resultSet, currentConnection));
			}
		}
		finally {
			currentConnection.close();
		}
		
		return posts;
	}

	@Override
	void save(Post post, Connection connection) throws Exception {
		PreparedStatement insertStatement = connection.prepareStatement(SQL_INSERT, Statement.RETURN_GENERATED_KEYS);
		insertStatement.setString(1, post.getTitle());
		insertStatement.setString(2, post.getContent());
		insertStatement.executeUpdate();
		
		ResultSet resultSet = insertStatement.getGeneratedKeys();
		if (resultSet.next()) {
			Long key = resultSet.getLong(1);
			post.setId(key);
			
			Set<Comment> comments = post.getComments();
			if (comments != null) {
				for(Comment comment : comments) {
					comment.setPostId(key);
					commentDao.save(comment, connection);
				}
			}
		}
		else {
			throw new DaoException("Cannot get inserted post id!");
		}
	}

	@Override
	void update(Post post, Connection connection) throws Exception {
		PreparedStatement updateStatement = connection.prepareStatement(SQL_UPDATE);
		updateStatement.setString(1, post.getTitle());
		updateStatement.setString(2, post.getContent());
		updateStatement.setLong(3, post.getId());
		updateStatement.executeUpdate();
		
		Set<Comment> postComments = post.getComments();
		if (postComments != null) {
			for (Comment comment : postComments) {
				comment.setPostId(post.getId());
				if (comment.getId() != null) {
					commentDao.update(comment, connection);
				}
				else {
					commentDao.save(comment, connection);
				}
			}
		}
	}
	
}