package org.codeproject.dao.test;

import static java.lang.System.out;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;

import org.apache.commons.lang3.RandomStringUtils;
import org.codeproject.dao.DatabaseConnectionMgr;
import org.junit.BeforeClass;
import org.junit.Test;

public class TestDatabase {

	@BeforeClass
	public static void setUpOnce() throws Exception {
		Connection connection = DatabaseConnectionMgr.getConnection();
				
		try {
			Statement statement = connection.createStatement();
			statement.execute("DROP TABLE comment IF EXISTS");
			statement.execute("DROP TABLE post IF EXISTS");
			statement.execute("CREATE TABLE post (id INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, title VARCHAR(150), content LONGVARCHAR, creationDate TIMESTAMP, lastModified TIMESTAMP)");
			statement.execute("CREATE TABLE comment(id INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY, postId INTEGER, content LONGVARCHAR, FOREIGN KEY (postId) REFERENCES post(id) ON DELETE CASCADE, creationDate TIMESTAMP, lastModified TIMESTAMP)");
			//add update (lastModified) and insert (creationDate) trigger
			for (String tableName : new String[] {"post", "comment"}) {
				statement.execute(String.format("CREATE TRIGGER t_%s_update BEFORE UPDATE ON %s REFERENCING NEW AS newrow FOR EACH ROW BEGIN ATOMIC SET newrow.lastModified = CURRENT_TIMESTAMP; END", tableName, tableName));
				statement.execute(String.format("CREATE TRIGGER t_%s_insert BEFORE INSERT ON %s REFERENCING NEW AS newrow FOR EACH ROW BEGIN ATOMIC SET newrow.creationDate = CURRENT_TIMESTAMP; END", tableName, tableName));
			}
		}
		finally {
			connection.close();
		}
	}
	
	private String addPost() throws Exception {
		Connection connection = DatabaseConnectionMgr.getConnection();
		
		String namePart = RandomStringUtils.randomAscii(10);
		try {
			PreparedStatement statement = connection.prepareStatement("INSERT INTO post(title, content) VALUES(?,?)");
			statement.setString(1, String.format("test-title-%s", namePart));
			statement.setString(2, String.format("test-content-%s", namePart));
	
			statement.execute();
		}
		finally {
			connection.close();
		}
		
		return namePart;
	}

	@Test
	public void insert() throws Exception {
		addPost();
	}

	@Test
	public void select() throws Exception {
		String namePart = addPost();
		int size = 0;
		
		Connection connection = DatabaseConnectionMgr.getConnection();
		try
		{
			PreparedStatement selectStatement = connection.prepareStatement("SELECT * FROM post WHERE title LIKE ?");
			selectStatement.setString(1, "%" + namePart);
	
			ResultSet resultSet = selectStatement.executeQuery();
	
			assertNotNull(resultSet);
	
			while (resultSet.next()) {
				out.println(resultSet.getString("title"));
				size++;
			}
		}
		finally {
			connection.close();
		}

		assertTrue(size > 0);
	}
	
}