#include <stdio.h>
#include <string.h>
#include <malloc.h>
#define FALSE (1==0)
#define TRUE  (1==1)

typedef struct Vertex {
	int index;
	struct Vertex *next;
	struct Vertex *parent;
	char *data;
	int depth;
} Vertex;

typedef struct Tree {
	char size;
	char number_jugs;
	Vertex *start;
	Vertex *current;
	Vertex *last;
} Tree;

char capacities[] = {8, 5, 3};
char start_values[] = {8, 0, 0};

Vertex *vertex_new(int index, char config[], Vertex *parent, int depth, char size) {
	Vertex *vertex = malloc(sizeof(Vertex));
	vertex->index = index;
	vertex->data = malloc(size);
	vertex->next = NULL;
	vertex->parent = parent;
	vertex->depth = depth;
	memcpy(vertex->data,config,size);
	return vertex;
}

unsigned char equal_states(char state1[], char state2[], char number_jugs) {
	int i;
	for (i=0;i<number_jugs;i++)
		if (state1[i] != state2[i]) 
			return FALSE;
	return TRUE;
}

void add_new_vertices(Tree *tree) {

	Vertex *new;
	Vertex *check;
	char *vertex_temp;
	int from,to;
	unsigned char equal;
	
	vertex_temp = malloc(tree->size);

	for (from = 0; from < tree->number_jugs; from++) {
		if ((tree->current)->data[from] == 0 && capacities[from] != 0) continue;
		for (to = 0; to < tree->number_jugs; to++) {
			if (from == to) continue;
			memcpy (vertex_temp, (tree->current)->data, tree->number_jugs);
			if (capacities[from] == 0) {
				vertex_temp[to] = capacities[to];
			}			
			else if (capacities[to] == 0) {
				vertex_temp[from] = 0;
			}
			else if (vertex_temp[from] + vertex_temp[to] > capacities[to]) {
				vertex_temp[from] = vertex_temp[from] - (capacities[to] - vertex_temp[to]);
				vertex_temp[to]   = capacities[to];
			}
			else {
				vertex_temp[to] = vertex_temp[to] + vertex_temp[from];
				vertex_temp[from] = 0;
			}
			
			check = tree->start;
			equal = FALSE;
			while(TRUE) {
				if (equal_states (check->data, vertex_temp, tree->number_jugs)) {
					equal = TRUE;
					break;
				}
				check = check->next;
				if (check == NULL) {
					break;
				}
			}
			if (equal) 
				continue;

			new = vertex_new(((tree->last)->index)+1, vertex_temp, tree->current,((tree->current)->depth)+1, tree->size);
			tree->last->next = new;
			tree->last = new;
		}
	}
	free (vertex_temp);
	return;
}

void print_tree(Tree tree) {
	int i;
    Vertex *vertex;
    vertex = tree.start;
    
    while (TRUE) {
    	printf("Vertex %d Parent %d State [ ",vertex->index,(vertex->parent == NULL) ? -1 : vertex->parent->index);
	    for (i=0;i<tree.size;i++)
		    printf("%d ",vertex->data[i]);
    	printf("] Depth %d\n",vertex->depth);
    	if (vertex->next == NULL) break;
    	vertex = vertex->next;
    }
	return;
}

void free_heap(Vertex *start) {
	Vertex *vertex;
	Vertex *vertex_next;
	vertex = start;
	vertex_next = start->next;
	while (vertex != NULL) {
		vertex_next = vertex->next;
		free(vertex->data);
		free(vertex);
		vertex = vertex_next;
	}
	return;
}

int main(int argc, char *argv[]) {

	Tree tree;
	Vertex *start;
	
	start = vertex_new(0, start_values, NULL, 0, sizeof(capacities));
	
	tree.start = start;
	tree.current = start;
	tree.last = start;
    tree.number_jugs = sizeof(capacities) / sizeof(char);
    tree.size = sizeof(capacities);
        
	while (TRUE) {
		add_new_vertices(&tree);
        tree.current = (tree.current)->next;
        if (tree.current == NULL)
        	break;
	}

    print_tree(tree);
	free_heap(tree.start);
	return 0;
}
