#!/usr/bin/runhugs

-- Entry: Data structure of tree entry
data Entry = Entry { index :: Int,
                     parent :: Int,
                     state :: [Integer],
                     depth :: Int }
            deriving (Show)

-- into: Set list element at position 'to' to 'value'
into :: Int -> Integer -> [Integer] -> [Integer]
into to value list = (take to list) ++ [value] ++ (drop (to+1) list)

-- pour: Pour jug 'from' into jug 'to', jugs are in 'state', with jugs 'capacities'
pour :: (Int,Int) -> [Integer] -> [Integer] -> [Integer]
pour (from, to) s c
                    | (from == to)              = s                             -- Fill jug into itself
                    | (c !! from) == 0 = into to (c !! to) s                    -- Fill jug from infinite source
                    | (c !! to)   == 0 = into from 0 s                          -- Empty jug into infinite source
                    | (s !! from + s !! to) > (c !! to) = into to (c !! to) (into from (s !! from - (c !! to - s !! to) ) s)
                    | otherwise = into from 0 (into to (s !! from + s !! to) s)

-- allpossible: Get all possible new states from specific filling state
allpossible :: [Integer] -> [Integer] -> [[Integer]]
allpossible s c  = [pour (x,y) s c | x <- [0..(length s)-1], y <- [0..(length s)-1], not (x == y)]

startNode = Entry {index = 0, parent = (-1), state = [8,0,0], depth = 0}

-- makeTree: construct jugs solution tree by breadth-first search
makeTree :: Int -> [Entry] -> [Integer] -> [Entry]
makeTree counter entries cap
           | (counter >= length(entries)) = [(entries!!c){index = c}| c <- [0..(length entries)-1]]
           | otherwise = makeTree (counter+1) (entries ++ [startNode{parent=counter, depth = nextdepth +1, state = x} | x <- nextentries, not (elem x allentries)]) cap
                where allentries = [state x | x <- entries]
                      nextelement = [state x | x <- entries]!!counter
                      nextdepth = [depth x | x <- entries]!!counter
                      nextentries = allpossible nextelement cap

--- jugs: Initiate makeTree with startValue, capacities
jugs :: [Integer] -> [Integer] -> [Entry]
jugs startvalue capacities = makeTree 0 [startNode{state = startvalue}] capacities

--- printAll: Print one list element per line
printAll :: Show a => [a] -> IO ()
printAll [] = return ()
printAll (x:xs) = do print x
                     printAll xs
                     
main = printAll (jugs [8,0,0] [8,5,3])

