#include <stdio.h>
#include <stdlib.h>


/* 
 * DQueue:      left                             right
 * -------        |                                 |
 *                v                                 v
 *           +---------+      +---------+      +---------+      
 *  NULL <---|  left   |  <---|  left   |  <---|  left   | 
 *           +---------+      +---------+      +---------+      
 *           |  right  |--->  |  right  |--->  |  right  |---> NULL
 *           +---------+      +---------+      +---------+
 * 
 */


typedef struct _dqueue_item {
  struct _dqueue_item *left;
  struct _dqueue_item *right;
  void *data;
} DQueueItem;


typedef struct {
  struct _dqueue_item *left;
  struct _dqueue_item *right;
} DQueue;


void insert_left (DQueue *D, void *data)
{
  DQueueItem *item = malloc (sizeof (DQueueItem));

  item->data = data;
  item->left = NULL;

  item->right = D->left;

  if (D->left != NULL)
    {
      /* not empty */
      D->left->left = item;
    }
  else
    {
      /* empty */
      D->right = item;
    }
  
  D->left = item;
}


void *remove_left (DQueue *D)
{
  void *data = NULL;
  DQueueItem *item = D->left;

  if (item != NULL)
    {
      /* not empty */
      data = item->data;
      D->left = item->right;

      if (D->left != NULL)
        {
          /* still not empty */
          D->left->left = NULL;
        }
      else
        {
          /* empty now */
          D->right = NULL;
        }

      free (item);
    }
  
  return data;
}


void insert_right (DQueue *D, void *data)
{
  DQueueItem *item = malloc (sizeof (DQueueItem));

  item->data = data;
  item->right = NULL;

  item->left = D->right;

  if (D->right != NULL)
    {
      /* not empty */      
      D->right->right = item;
    }
  else
    {
      /* empty */
      D->left = item;
    }
  
  D->right = item;
}


void *remove_right (DQueue *D)
{
  void *data = NULL;
  DQueueItem *item = D->right;

  if (item != NULL)
    {
      /* not empty */
      data = item->data;
      D->right = item->left;

      if (D->right != NULL)
        {
          /* still not empty */
          D->right->right = NULL;
        }
      else
        {
          /* empty now */
          D->left = NULL;
        }

      free (item);
    }
  
  return data;
}


void test (void)
{
  DQueue D = { NULL, NULL };

  printf ("remove_left:\n");

  insert_right (&D, "F");
  insert_left (&D, "E");
  insert_left (&D, "D");
  insert_right (&D, "G");
  insert_left (&D, "C");

  char *s;
  while ((s = remove_left (&D)) != NULL)
    printf ("%s\n", s);

  printf ("---\n");
  

  printf ("remove_right:\n");

  insert_right (&D, "F");
  insert_left (&D, "E");
  insert_left (&D, "D");
  insert_right (&D, "G");
  insert_left (&D, "C");

  while ((s = remove_right (&D)) != NULL)
    printf ("%s\n", s);

  printf ("---\n");

}


int main(int argc, char *argv[])
{

  test ();
  
  return 0;
}
