#include <glib.h>
#include <string.h>
#include <stdlib.h>

typedef struct State {
    gboolean reached;
    struct State *parent;
    GList *next;
    guchar config[];
} State;

guchar capacities[] = {8, 3, 5};
guchar start_config[] = {8, 0, 0};
guchar goal_value = 6;
gsize n_caps = sizeof (capacities);

GQueue *queue;
GHashTable *nodes = NULL;

guint state_hash (guchar *config) {
    return config[0];
}

gboolean config_equal (guchar *a, guchar *b) {
    if (!memcmp (a, b, n_caps))
        return TRUE;
    else return FALSE;
}

void dump_config (guchar *config) {
    g_print ("config [ ");
    int i;
    for (i=0; i<n_caps; ++i)
        g_print ("%d ", config[i]);
    g_print ("]\n");
}

State *state_new (guchar *config) {
    State *state = g_malloc (sizeof (State) + n_caps);
    state -> reached = FALSE;
    state -> parent = NULL;
    state -> next = NULL;
    memcpy (state->config, config, n_caps);

    return state;
}

void add_node (State *state) {
    g_hash_table_insert (nodes, state->config, state);
    g_queue_push_head (queue, state);

#ifdef VERBOSE
    g_print ("add_node ");
    dump_config (state->config);
#endif
}

guchar *swap (State *state, gint from, gint to) {
    gint delta  = MIN (capacities[to] - state->config[to], state->config[from]);
    if (delta <= 0) {
        return NULL;
    }
    guchar *config = g_malloc (n_caps);

    memcpy (config, state->config, n_caps);
    config[from] -= delta;
    config[to] += delta;

    return config;
}

void check_state (State *state) {
    int from, to;
    guchar *config;

    for (from=0; from < n_caps; ++from)
        for (to=0; to < n_caps; ++to)
            if (from != to)
                if (config = swap (state, from, to)) {
                    State *target = g_hash_table_lookup (nodes, config);
                    if (! target) {
                        target = state_new (config);
                        add_node (target);
                    }
                    g_free (config);

                    state->next = g_list_append (state->next, target);
                }
}

void print_solution (State *state) {
    while (state) {
        dump_config (state->config);
        state = state->parent;
    }
}

void check_goal (State *state) {
    int i;
    for (i=0; i<n_caps; ++i)
        if (state->config[i] == goal_value) {
            g_print ("solution:\n");
            print_solution (state);
            exit (0);
        }
}

int main (int arc, char *argv[]) {
    nodes = g_hash_table_new ((GHashFunc) state_hash,
                              (GEqualFunc) config_equal);
    queue = g_queue_new ();

    State *start = state_new (start_config);
    add_node (start);

    while (! g_queue_is_empty (queue)) {
        State *state = g_queue_pop_head (queue);
        check_state (state);
    }

    /* BFS */
    g_queue_push_tail (queue, start);
    start->reached = TRUE;

    while (! g_queue_is_empty (queue)) {
        State *state = g_queue_pop_head (queue);
        GList *cur;
        for (cur = state->next; cur != NULL; cur = cur->next) {
            State *child = cur->data;
            if (child->reached)
                continue;
            child->reached = TRUE;
            child->parent = state;
            g_queue_push_tail (queue, child);
            check_goal (child);
        }
    }
    g_print ("no solution\n");

    return 0;
}
