#include <glib.h>
#include <string.h>

#include "json-util.h"
#include "json-tree-direct-storage.h"

#include "shs-node.h"


const gchar *get_label (ShsNode *n);
gint get_width (gpointer node);

void print_label (ShsNode *node, gpointer data);
void pretty_print (ShsNode *node);

GString *pretty_string (ShsNode *node);
GString *pretty_line (ShsNode *node);


static void mk_tree (gpointer t_node, JsonNode *j_node) {
  GList *children;
  GList *j_child;
  gpointer t_child;

  children = json_tree_get_children (j_node);
  for (j_child=children; j_child!=NULL; j_child=j_child->next) {
    t_child = node_new (j_child->data);
    node_append (t_node, t_child);
    mk_tree (t_child, j_child->data);
  }

  g_list_free (children);
}

const gchar *get_label (ShsNode *n) {
  return json_node_get_string (json_object_get_member (json_node_get_object (n->data), ID_MARKER));
}

int get_width (gpointer node) {
  /* GNode *c; */
  gpointer c;
  int w;

  if (node == NULL)
    return -1;

  w = 0;

  c = node_first_child (node);
  while (c != NULL) {
    w += get_width (c);
    c = node_next_sibling (c);
  }
  
  return MAX (w, strlen (get_label (node)) + 1);
}


GString *pretty_string (ShsNode *node) {
  GString *string;

  string = g_string_new ("");
  
  while (node != NULL) {
    ShsNode *n = node;
    int w = get_width (n->parent);
    while (n != NULL) {
      g_string_append_printf (string, "%s", get_label (n));
      if (n->next != NULL)
        g_string_append_printf (string, "-");
      else
        g_string_append (string, " ");
      g_string_append_printf (string, "%*s", get_width (n) - strlen (get_label (n)) - 1, "");
      w -= get_width (n);

      if (n->next == NULL) {
        if (w > 0)
          g_string_append_printf (string, "%*s", w, "");
        n = node_next (n);
      }
      else
        n = n->next;
    }
    g_string_append (string, "\n");
    node = node->children;
  }

  return string;
}


void print_label (ShsNode *node, gpointer data) {
  int i;
  for (i=0; i<node_depth (node); ++i)
    g_print ("  ");

  g_print ("%s\n", get_label (node));
}


int main (int argc, char *argv[]) {
  JsonNode *text;
  ShsNode *root;
  GString *s;

  s = NULL;
  g_type_init ();

  text = json_read ("data/text-3.json");

  root = node_new (text);
  mk_tree (root, text);

  node_traverse_pre_order (root, print_label, NULL);

  s = pretty_string (root);
  g_print ("%s", s->str);
  g_string_free (s, TRUE);
  
  return 0;
}

