/*
 * ShsBox
 */

#include "shs-box.h"

#define SHS_BOX_GET_PRIVATE(obj) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), SHS_TYPE_BOX, ShsBoxPrivate))

struct _ShsBoxPrivate {
  
  gchar* collection;
};

G_DEFINE_TYPE (ShsBox, shs_box, G_TYPE_OBJECT)

enum {
  PROP_0,
  PROP_COLLECTION,
  N_PROPERTIES
};


void shs_box_set_collection (ShsBox *self, const gchar* value) {
  g_return_if_fail (SHS_IS_BOX(self));

  if (g_strcmp0 (self->priv->collection, value) != 0) {
    g_free (self->priv->collection);
    self->priv->collection = g_strdup (value);
    g_object_notify (G_OBJECT (self), "collection");
  }
}


static void shs_box_set_property (GObject *gobject, guint prop_id,
                                        const GValue *value, GParamSpec *pspec) {
  ShsBox  *self;
  self = SHS_BOX (gobject);
  
  switch (prop_id) {
    case PROP_COLLECTION:
      shs_box_set_collection (self, g_value_get_string (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}


gchar* shs_box_get_collection (ShsBox *self) {
  g_return_val_if_fail (SHS_IS_BOX(self), NULL);
  return self->priv->collection;
}


static void shs_box_get_property (GObject *gobject, guint prop_id, GValue *value, GParamSpec *pspec) {
  ShsBox  *self;
  self = SHS_BOX (gobject);
  
  switch (prop_id) {
    case PROP_COLLECTION:
      g_value_set_string (value, shs_box_get_collection (self));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
      break;
  }
}


static void shs_box_dispose (GObject *gobject) {
  ShsBox *self;
  self = SHS_BOX (gobject);
  
  /* unref all references _once_!
     (might be called multiple times)
     
     if (self->priv->object) {
     g_object_unref (self->priv->object);
     self->priv->object = NULL;
     }
  */
  
  /* Chain up to the parent class */
  G_OBJECT_CLASS (shs_box_parent_class)->dispose (gobject);
}


static void shs_box_finalize (GObject *gobject) {
  ShsBox *self;
  self = SHS_BOX (gobject);

  g_free (self->priv->collection);
  
  /* Chain up to the parent class */
  G_OBJECT_CLASS (shs_box_parent_class)->finalize (gobject);
}


static void shs_box_class_init (ShsBoxClass *klass) {
  GObjectClass *gobject_class;
  gobject_class = G_OBJECT_CLASS (klass);
  
  /* install methods */
  klass->do_action = shs_box_do_action;
  
  gobject_class->dispose = shs_box_dispose;
  gobject_class->finalize = shs_box_finalize;
  
  gobject_class->set_property = shs_box_set_property;
  gobject_class->get_property = shs_box_get_property;
  
  /* install properties */
  
  g_object_class_install_property (gobject_class, PROP_COLLECTION,
    g_param_spec_string ("collection", "nick", "blurb",
                         NULL, G_PARAM_READWRITE));
  g_type_class_add_private (klass, sizeof (ShsBoxPrivate));
}


static void shs_box_init (ShsBox *self) {
  self->priv = SHS_BOX_GET_PRIVATE (self);
  
  /* initialize all public and private members to reasonable default values. */
  self->priv->collection = NULL;
}


/* VIRTUAL METHODS */

void shs_box_do_action (ShsBox *self) {
}
