/* This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <stdio.h>
#include <string.h>

char s_table [] = { 0, 7, 0,   5, 9, 2,   4, 8, 0,
                    0, 2, 0,   0, 0, 0,   0, 7, 5,
                    8, 5, 0,   6, 0, 7,   0, 0, 0,

                    2, 0, 5,   0, 0, 0,   3, 0, 0,
                    6, 0, 0,   0, 0, 0,   0, 0, 7,
                    7, 0, 3,   0, 0, 0,   8, 0, 2,

                    0, 0, 0,   3, 0, 5,   0, 0, 8,
                    4, 0, 0,   0, 0, 0,   0, 0, 0,
                    5, 0, 2,   1, 6, 0,   7, 9, 0
};

int solutions = 0;


void sudoku_print (char* table)
{
  int i;

  for (i=1; i<=81; ++i)
    {
      printf ("%d", table[i-1]);
      if (i % 9 == 0)
        printf ("\n");
      if ((i % 3 == 0) && (i % 9 != 0))
        printf (" ");
      if ((i % 27 == 0) && (i % 81 !=0))
        printf ("\n");
    }
  printf ("\n---\n");
}


int next_cell (char *table, int index)
{
  while (index < 81)
    {
      if (table[index] == 0)
        return index;
      ++index;
    }
  return -1;
}


void sudoku_check (char *table, int index, char *allowed)
{
  int row = index / 9;
  int column = index % 9;
  int box_row = (row / 3) * 3;
  int box_column = (column / 3) * 3;

  int i;
  char value;

  memset (allowed, 1, 10);

  for (i=0; i<9; ++i)
    {
      /* check row */
      value = table[(9 * row) + i];
      allowed[value] = 0;
      /* check column */  
      value = table[(9 * i) + column];
      allowed[value] = 0;
      /* check box */
      value = table[(9 * (box_row + (i / 3))) + box_column + (i % 3)];
      allowed[value] = 0;      
    }
}


void sudoku_solve (char *old_table, int index)
{
  char table[81];
  char allowed[10];

  int i;

  index = next_cell (old_table, index);
  if (index == -1)
    {
      //sudoku_print (old_table);
      ++solutions;
      return;
    }

  memcpy (table, old_table, 81);

  sudoku_check (table, index, allowed);

  for (i=1; i<10; ++i)
    {
      if (allowed[i] == 1)
        {
          table[index] = i;
          sudoku_solve (table, index);
        }
    }
}


int main(int argc, char *argv[])
{
  sudoku_solve (s_table, 0);
  printf ("%d solutions\n", solutions);
  return 0;
}
