# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

use strict;

my @s_table = ( 0, 7, 0,   5, 9, 2,   4, 8, 0,
                0, 2, 0,   0, 0, 0,   0, 7, 5,
                8, 5, 0,   6, 0, 7,   0, 0, 0,

                2, 0, 5,   0, 0, 0,   3, 0, 0,
                6, 0, 0,   0, 0, 0,   0, 0, 7,
                7, 0, 3,   0, 0, 0,   8, 0, 2,

                0, 0, 0,   3, 0, 5,   0, 0, 8,
                4, 0, 0,   0, 0, 0,   0, 0, 0,
                5, 0, 2,   1, 6, 0,   7, 9, 0
    );

my $solutions = 0;

sub next_cell
{
    my ($table, $index) = @_;
    while ($index < 81)
    {
        if ($table->[$index] == 0)
        {
            return $index;
        }
        ++$index;
    }
    return -1;
}


sub sudoku_check
{
    my ($table, $index) = @_;

    my $row = int ($index / 9);
    my $column = int ($index % 9);
    my $box_row = int ($row / 3) * 3;
    my $box_column = int ($column / 3) * 3;

    my $value;

    my @allowed = (1, 1, 1, 1, 1, 1, 1, 1, 1, 1);
    
    for (my $i=0; $i<9; ++$i)
    {
        # check row
        $value = $table->[(9 * $row) + $i];
        $allowed[$value] = 0;
        # check column
        $value = $table->[(9 * $i) + $column];
        $allowed[$value] = 0;
        # check box
        $value = $table->[(9 * ($box_row + int ($i / 3))) + $box_column + int ($i % 3)];
        $allowed[$value] = 0;      
    }
    return @allowed;
}


sub sudoku_solve
{
    my ($old_table, $index) = @_;

    $index = next_cell ($old_table, $index);
    if ($index == -1)
    {
        ++$solutions;
        return;
    }

    my @table = @$old_table;

    my @allowed = sudoku_check (\@table, $index);

    for (my $i=1; $i<10; ++$i)
    {
        if ($allowed[$i] == 1)
        {
            $table[$index] = $i;
            sudoku_solve (\@table, $index);
        }
    }
}


sudoku_solve (\@s_table, 0);
printf ("%d solutions\n", $solutions);

