# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


#==== IMPORTS ============================

import math

import Blender
from Blender import NMesh	

def plot (function):
	mesh = NMesh.GetRaw ()

	for i in range (0, N_VERTS):
		for j in range (0, N_VERTS):
			x = (WIDTH * i) / (N_VERTS - 1)
			y = (HEIGHT * j) / (N_VERTS - 1)
			x -= WIDTH/2
			y -= HEIGHT/2
			z = function (x, y)

			vertex = NMesh.Vert (x, y, z)
			mesh.verts.append (vertex)

	for i in range (0, N_VERTS - 1):
		for j in range (0, N_VERTS -1):
			f = NMesh.Face ()
			
			f.v.append (mesh.verts[i*N_VERTS + j])
			f.v.append (mesh.verts[(i + 1)*N_VERTS + j])
			f.v.append (mesh.verts[(i + 1)*N_VERTS + j +1])
			f.v.append (mesh.verts[i*N_VERTS + j + 1])
			
			mesh.faces.append (f)

	NMesh.PutRaw (mesh, "Function", 1)
	Blender.Redraw ()
	

#==== GLOBALS ============================

WIDTH = 20.0
HEIGHT = 20.0
N_VERTS = 100	


#==== FUNCTIONS ==========================

def f (x, y):
	x /= 5
	y /= 5
	return x**2 - y**2

def g (x, y):
	return math.sin (x) + math.sin (y)

def h (x, y):
	d = math.sqrt (x**2 + y**2)
	return math.sin(d)

def j (x, y):
	d1 = math.sqrt ((x-1)**2 + y**2)
	d2 = math.sqrt ((x+1)**2 + y**2)
	return math.sin (d1) + math.sin (d2)

#==== MAIN ===============================

plot (f)
